# -*- encoding: utf-8 -*

from django.core.files.storage import FileSystemStorage
from django.db import models
import datamaster_modeles.models as ref
import settings

### CONSTANTES ###
# HELP_TEXT
HELP_TEXT_NB_DEPENDANT = "Le nombre de personnes à charge"
HELP_TEXT_TAGS_ACCEPTES = "Pour le texte, les variables disponibles sont : \
                            {{ nom_candidat }} {{ prenom_candidat }} \
                            {{ offre_emploi }}. Ces champs seront \
                            automatiquement remplacés par les informations de \
                            chaque candidat."
HELP_TEXT_EN_AFFICHAGE = "En cochant cette case, l'offre d'emploi sera affichée\
                         sur le site institutionnel dès qu'elle aura le statut \
                        'En diffusion' et ce, jusqu'à la date limite."

STATUT_OFFRE_EMPLOI_CHOICES = (
    ('NOUV', 'Nouveau'),
    ('AFFI', 'En diffusion'),
    ('EVAL', 'En évaluation des candidatures'),
    ('ENTR', 'En entrevue'),
    ('TERM', 'Terminé'),
)

# CANDIDAT
GENRE_CHOICES = (
    ('M', 'Homme'),
    ('F', 'Femme'),
)

SITUATION_CHOICES = (
    ('C', 'Célibataire'),
    ('M', 'Marié'),
    ('F', 'Conjoint de fait'),
    ('P', 'Pacsé'),
)

STATUT_CHOICES = (
    ('NOUV', 'Nouveau'),
    ('REC', 'Recevable'),
    ('SEL', 'Sélectionné pour entrevue'),
    ('REF', 'Non retenu'),
    ('ACC', 'À recruter'),
)

# PIECE CANDIDAT
TYPE_PIECE_CHOICES = (
    ('CV','CV'),
    ('LET','Lettre'),
    ('AUT','Autre'),
)

# Abstracts
class Metadata(models.Model):
    """
    Méta-données AUF.
    Metadata.actif = flag remplaçant la suppression.
    actif == False : objet réputé supprimé.
    """
    actif = models.BooleanField(default=True)
    date_creation = models.DateField(auto_now_add=True, )
    
    class Meta:
        abstract = True

class OffreEmploi(Metadata):
    est_affiche = models.BooleanField(default=False, 
                        verbose_name="À afficher sur le site institutionnel",
                        help_text=HELP_TEXT_EN_AFFICHAGE)
    statut = models.CharField(max_length=4, choices=STATUT_OFFRE_EMPLOI_CHOICES,
                                default='NOUV')
    date_limite = models.DateField(verbose_name="Date limite",)
    nom = models.CharField(max_length=255)
    resume = models.TextField(verbose_name="Résumé", blank=True)
    description = models.TextField()
    poste = models.CharField(max_length=255)
    poste_nom = models.CharField(max_length=255)
    region = models.ForeignKey(ref.Region, db_column='region', 
                verbose_name="Région")
    bureau = models.ForeignKey(ref.Bureau, db_column='bureau', )
    duree_affectation = models.CharField(max_length=255, 
                        verbose_name="Durée de l'affectation")
    renumeration = models.CharField(max_length=255,
                    verbose_name='Rénumération')
    debut_affectation = models.DateField(verbose_name="Début de l'affectation",)
    lieu_affectation = models.ForeignKey(ref.Implantation,  
                        db_column='implantation', 
                        verbose_name="Lieu d'affectation")

    class Meta:
        db_table = 'emploi_offreemploi'
        verbose_name_plural = "offres d'emploi"

    def __unicode__(self):
        return '%s [%s]' % (self.nom, self.id)

class Candidat(Metadata):   
    statut = models.CharField(max_length=4, choices=STATUT_CHOICES, 
                default='NOUV')
    offre_emploi = models.ForeignKey('OffreEmploi', db_column='offre_emploi',
                    related_name='+')
    prenom = models.CharField(max_length=255, verbose_name='Prénom', )
    nom = models.CharField(max_length=255)
    genre = models.CharField(max_length=1, choices=GENRE_CHOICES)
    nationalite = models.ForeignKey(ref.Pays, 
                    db_column='nationalite', related_name='+',
                    verbose_name='Nationalité')
    situation_famille = models.CharField(max_length=1, 
                        choices=SITUATION_CHOICES, 
                        verbose_name='Situation familiale', )
    nombre_dependant = models.IntegerField(verbose_name='Nombre de dépendant',
                        help_text=HELP_TEXT_NB_DEPENDANT, )
    niveau_diplome = models.CharField(max_length=255,
                        verbose_name='Niveau du diplôme')
    employeur_actuel = models.CharField(max_length=255, )
    poste_actuel = models.CharField(max_length=255, )
    domaine_professionnel = models.CharField(max_length=255, )
    telephone = models.CharField(max_length=255, verbose_name='Téléphone', )
    email = models.EmailField(max_length=255, verbose_name = 'Courriel', )

    # Adresse
    adresse = models.CharField(max_length=255)
    ville = models.CharField(max_length=255)
    etat_province = models.CharField(max_length=255, 
                    verbose_name="État/Province", blank=True)
    code_postal = models.CharField(max_length=255, blank=True)
    pays = models.ForeignKey(ref.Pays, db_column='pays',
            related_name='+')

    class Meta:
        db_table = 'emploi_candidat'

    def pieces_jointes(self):
        return CandidatPiece.objects.filter(candidat=self) 
    pieces_jointes.allow_tags = True    

    def __unicode__(self):
        return '%s %s [%s]' % (self.nom, self.prenom, self.id)


# Upload de fichiers
storage_prive = FileSystemStorage(settings.OE_PRIVE_MEDIA_ROOT, 
                            base_url=settings.OE_PRIVE_MEDIA_URL)

def candidat_piece_dispatch(instance, filename):
    path = u'%s/%s/%s' % ('emplois', instance.candidat.id, filename)
    return path

class CandidatPiece(models.Model):
    candidat = models.ForeignKey(Candidat, db_column='candidat',
                related_name='candidat_piece') 
    nom = models.CharField(max_length=3, choices=TYPE_PIECE_CHOICES)
    path = models.FileField(verbose_name="Fichier", 
                        upload_to=candidat_piece_dispatch, storage=storage_prive)

    class Meta:
        db_table = 'emploi_pieces'
        verbose_name = "pièce jointe"
        verbose_name_plural = "pièces jointes"

    def __unicode__(self):
        return '%s' % (self.nom)
